@@ -17,7 +17,7 @@
 #  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #  MA 02110-1301, USA.
 #
-#  version: 20220819120703
+#  version: 20220821163151
 
 from json import dumps
 import mimetypes
@@ -27,11 +27,11 @@ from io import BytesIO as IO
 from datetime import datetime, timedelta
 from dateutil import tz
 from http import HTTPStatus
-from http.server import BaseHTTPRequestHandler, ThreadingHTTPServer
+from http.server import BaseHTTPRequestHandler, HTTPServer
+from urllib.parse import urlparse, quote, unquote_plus
 import logging
 
 from .Database import Measurement
-from .Graph import graph
 from .Utils import DatetimeEncoder
 
 
@@ -48,10 +48,6 @@ class InterceptorHandlerFactory:
                 r"^/sensorlog\?hum=(?P<humidity>\d+(\.\d+)?)\&temp=(?P<temperature>\d+(\.\d+)?)\&id=(?P<stationid>[a-z01-9-]+)$",
                 re.IGNORECASE,
             )
-            graphpattern = re.compile(
-                r"^/graph\?id=(?P<stationid>[a-z01-9-]+)$",
-                re.IGNORECASE,
-            )
             htmlpattern = re.compile(
                 r"^/html(\?id=(?P<stationid>[a-z01-9-]+))?$",
                 re.IGNORECASE,
@@ -70,9 +66,27 @@ class InterceptorHandlerFactory:
                 re.IGNORECASE,
             )
             staticpattern = re.compile(
-                r"^/static/(?P<resource>.+)$",
+                r"^(/static/(?P<resource>.*))|/$",
                 re.IGNORECASE,
             )
+            faviconpattern = re.compile(r"^/favicon.ico$")
+
+            def send_response(self, code, message=None):
+                """Add the response header to the headers buffer and log the
+                response code.
+
+                Overridden to leave out Server header (leaks information)
+                """
+                self.log_request(code)
+                self.send_response_only(code, message)
+                # self.send_header('Server', self.version_string())
+                self.send_header("Date", self.date_time_string())
+
+            @staticmethod
+            def checkPath(path: Path):
+                for p in path.parts:
+                    if p in {".", ".."}:
+                        raise ValueError("relative paths are forbidden")
 
             @staticmethod
             def getTimeseries(db, stationid):
@@ -83,9 +97,30 @@ class InterceptorHandlerFactory:
                     mtime if mtime is not None else mark,
                 )
 
+            def common_headers(self):
+                self.send_header(
+                    "Content-Security-Policy",
+                    "default-src 'self'; script-src 'self' https://cdn.jsdelivr.net/npm/; object-src 'none'; base-uri 'self'; frame-ancestors 'self';",
+                )
+                self.send_header("X-Content-Type-Options", "nosniff")
+
             def do_GET(self):
                 logging.info(self.path)
                 try:
+                    if re.match(self.faviconpattern, self.path):
+                        filepath = Path(static_directory) / "favicon.ico"
+                        mime_type = mimetypes.guess_type(filepath)[0]
+                        try:
+                            with open(filepath, "rb") as f:
+                                b = f.read()
+                                self.send_response(HTTPStatus.OK)
+                                self.send_header("Content-type", mime_type)
+                                self.send_header("Content-Length", str(len(b)))
+                                self.end_headers()
+                                self.wfile.write(b)
+                                return
+                        except FileNotFoundError:
+                            self.send_response(HTTPStatus.NOT_FOUND)
                     if m := re.match(self.querypattern, self.path):
                         measurement = Measurement(
                             m.group("stationid"),
@@ -94,19 +129,6 @@ class InterceptorHandlerFactory:
                         )
                         db.storeMeasurement(measurement)
                         self.send_response(HTTPStatus.OK)
-                    elif m := re.match(self.graphpattern, self.path):
-                        now = datetime.now(tz=tz.tzlocal())
-                        yesterday = now - timedelta(1)
-                        f = IO(b"")
-                        graph(db, f, m.group("stationid"), yesterday, now)
-                        png = f.getvalue()
-                        self.send_response(HTTPStatus.OK)
-                        self.send_header("Content-type", "image/png")
-                        self.send_header("Content-Length", str(len(png)))
-                        self.end_headers()
-                        self.wfile.write(png)
-                        f.close()
-                        return
                     elif m := re.match(self.htmlpattern, self.path):
                         ms = db.retrieveLastMeasurement(m.group("stationid"))
                         mdivs = "\n".join(
@@ -140,6 +162,7 @@ class InterceptorHandlerFactory:
                         self.send_response(HTTPStatus.OK)
                         self.send_header("Content-type", "text/html")
                         self.send_header("Content-Length", str(len(html)))
+                        self.common_headers()
                         self.end_headers()
                         self.wfile.write(html)
                         return
@@ -186,6 +209,7 @@ class InterceptorHandlerFactory:
                         self.send_response(HTTPStatus.OK)
                         self.send_header("Content-type", "text/html")
                         self.send_header("Content-Length", str(len(html)))
+                        self.common_headers()
                         self.end_headers()
                         self.wfile.write(html)
                         return
@@ -216,17 +240,20 @@ class InterceptorHandlerFactory:
                         self.send_response(HTTPStatus.OK)
                         self.send_header("Content-type", "application/json")
                         self.send_header("Content-Length", str(len(json)))
+                        self.common_headers()
                         self.end_headers()
                         self.wfile.write(json)
                         return
-                    elif m := re.match(self.namespattern, self.path):
+                    elif m := re.match(self.namespattern, unquote_plus(self.path)):
                         if not self.path.startswith("/names"):
                             stationid = m.group("stationid")
                             name = m.group("name")
-                            result = db.names(stationid, name)
-                        names = db.names("*")
+                            names = db.names(stationid, name)
+                        else:
+                            names = db.names("*")
                         names = "\n".join(
-                            f"<tr><td>{n[0]}</td><td>{n[1]}</td></tr>" for n in names
+                            f'<tr><td>{s}</td><td>{n}</td><td><a href="/static/updatename.html?id={quote(s)}&name={quote(n)}">Change</a></td></tr>'
+                            for s, n in names.items()
                         )
                         self.send_response(HTTPStatus.OK)
                         html = f"""<html><body>
@@ -235,11 +262,18 @@ class InterceptorHandlerFactory:
                         """
                         self.send_header("Content-type", "text/html")
                         self.send_header("Content-Length", str(len(html)))
+                        self.common_headers()
                         self.end_headers()
                         self.wfile.write(bytes(html, "UTF-8"))
                         return
                     elif m := re.match(self.staticpattern, self.path):
-                        filepath = Path(static_directory) / m.group("resource")
+                        path = urlparse(m.group("resource")).path
+                        filepath = Path(static_directory) / (
+                            path.decode() if type(path) is bytes else path
+                        )  # TODO check encoding URLs are not UTF-8?
+                        InterceptorHandler.checkPath(filepath)
+                        if filepath.is_dir():
+                            filepath /= "index.html"
                         mime_type = mimetypes.guess_type(filepath)[0]
                         try:
                             with open(filepath, "rb") as f:
@@ -247,22 +281,23 @@ class InterceptorHandlerFactory:
                                 self.send_response(HTTPStatus.OK)
                                 self.send_header("Content-type", mime_type)
                                 self.send_header("Content-Length", str(len(b)))
+                                self.common_headers()
                                 self.end_headers()
                                 self.wfile.write(b)
                                 return
                         except FileNotFoundError:
                             self.send_response(HTTPStatus.NOT_FOUND)
                     else:
-                        self.send_response(HTTPStatus.FORBIDDEN)
+                        self.send_response_only(HTTPStatus.FORBIDDEN)
                 except Exception as e:
                     logging.exception(e)
-                    self.send_response(HTTPStatus.INTERNAL_SERVER_ERROR)
+                    self.send_response_only(HTTPStatus.INTERNAL_SERVER_ERROR)
                 self.end_headers()
 
         return InterceptorHandler
 
 
-class Interceptor(ThreadingHTTPServer):
+class Interceptor(HTTPServer):
     allow_reuse_address = True
 
     def __init__(self, server_address, db, static_directory):
