@@ -52,12 +52,12 @@ def finish(self):
 
 
 def date_time_string(self, timestamp=None):
-    """ Mocked date time string """
+    """Mocked date time string"""
     return "DATETIME"
 
 
 def version_string(self):
-    """ mock the server id """
+    """mock the server id"""
     return "BaseHTTP/x.x Python/x.x.x"
 
 
@@ -86,7 +86,7 @@ class TestInterceptor:
                         print(captured.out)
                         assert (
                             ihinstance.wfile.getvalue()
-                            == b"HTTP/1.0 200 OK\r\nServer: BaseHTTP/x.x Python/x.x.x\r\nDate: DATETIME\r\n\r\n"
+                            == b"HTTP/1.0 200 OK\r\nDate: DATETIME\r\n\r\n"
                         )
                         database.names(
                             stationid, "testroom1"
@@ -119,10 +119,7 @@ class TestInterceptor:
                         print(captured.out)
                         response = ihinstance.wfile.getvalue()
                         print(response)
-                        assert (
-                            response
-                            == b"HTTP/1.0 403 Forbidden\r\nServer: BaseHTTP/x.x Python/x.x.x\r\nDate: DATETIME\r\n\r\n"
-                        )
+                        assert response == b"HTTP/1.0 403 Forbidden\r\n\r\n"
                         database.names(
                             stationid, "testroom1"
                         )  # without a stationid mapping we never get anything back
@@ -156,10 +153,7 @@ class TestInterceptor:
                         print(captured.out)
                         response = ihinstance.wfile.getvalue()
                         print(response)
-                        assert (
-                            response
-                            == b"HTTP/1.0 500 Internal Server Error\r\nServer: BaseHTTP/x.x Python/x.x.x\r\nDate: DATETIME\r\n\r\n"
-                        )
+                        assert response == b"HTTP/1.0 500 Internal Server Error\r\n\r\n"
                         database.names(
                             stationid, "testroom1"
                         )  # without a stationid mapping we never get anything back
@@ -167,34 +161,6 @@ class TestInterceptor:
                         print(start, r)
                         assert len(r) == 0
 
-    def test_GET_GRAPH(self, database, capsys):
-        stationid = "graphid-123456"
-        interceptorhandler = InterceptorHandlerFactory.getHandler(database, "./static")
-
-        database.names(
-            stationid, "testroom1"
-        )  # without a stationid mapping we never get anything back
-        start = datetime.now()
-        database.storeMeasurement(Measurement(stationid, 10, 40))
-        with mock.patch.object(interceptorhandler, "finish", finish):
-            with mock.patch.object(
-                interceptorhandler, "date_time_string", date_time_string
-            ):
-                with mock.patch.object(
-                    interceptorhandler, "version_string", version_string
-                ):
-                    with mock.patch.object(interceptorhandler, "wbufsize", lambda: 1):
-                        start = datetime.now()
-                        request = MockRequest(
-                            b"/graph?id=%s" % bytes(stationid, "UTF-8")
-                        )
-                        ihinstance = interceptorhandler(
-                            request, ("127.0.0.1", 12345), "testserver.example.org"
-                        )
-                        captured = capsys.readouterr()
-                        print(captured.out)
-                        assert ihinstance.wfile.getvalue()[:15] == b"HTTP/1.0 200 OK"
-
     def test_GET_HTML_all(self, database, capsys):
         stationid = "htmlid-123456"
         interceptorhandler = InterceptorHandlerFactory.getHandler(database, "./static")
